//
//  BWTableDataSource.h
//  UberProject
//
//  Created by Brian Webster on Mon May 12 2003.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class BWIndexMap;

typedef struct _BWTDSFlags
{
    unsigned int	delegateWillDisplayCell:1;
    unsigned int	delegateShouldEditTableColumn:1;
    unsigned int	delegateShouldSelectRow:1;
    unsigned int	delegateShouldSelectTableColumn:1;
    unsigned int	delegateSelectionShouldChangeInTableView:1;
} BWTDSFlags;

@interface BWTableDataSource : NSObject <NSCoding>
{
	NSString*					sortColumnName;
	BOOL						sortAscending;
	IBOutlet NSTableView*		tableView;
	IBOutlet NSTableColumn*		defaultSortColumn;
	BOOL						sortingEnabled;
	BOOL						filteringEnabled;
	BWIndexMap*					currentSortMap;
	id							dataSource;
	id							delegate;
	NSMutableDictionary*		columnSortOrders;
	NSString*					autosaveName;
}

- (BOOL)sortingEnabled;
- (void)setSortingEnabled:(BOOL)newSortingEnabled;
- (NSTableView *)tableView;
- (BOOL)filteringEnabled;
- (void)setFilteringEnabled:(BOOL)newFilteringEnabled;
- (NSString*)autosaveName;
- (void)setAutosaveName:(NSString*)newAutoSaveName;

- (void)loadConfiguration;
- (void)saveConfiguration;

- (NSTableView *)tableView;
- (NSString *)sortColumnName;
- (BOOL)sortAscending;

- (void)setSortColumnName:(NSString*)newColumnName sortAscending:(BOOL)newValue;
- (void)reloadData;
- (void)sortData;
- (BWIndexMap*)calculateFilterMap;
- (void)highlightSortColumn;

- (id)dataSource;
- (void)setDataSource:(id)newDataSource;
- (id)delegate;
- (void)setDelegate:(id)newDelegate;

- (int)rowForDisplayRow:(int)displayRow;
- (NSArray*)selectedRows;

@end

@interface NSObject(BWTableDataSource)

- (int)numberOfRowsInTableDataSource:(BWTableDataSource*)source;
- (id)tableDataSource:(BWTableDataSource*)source objectValueForTableColumn:(NSTableColumn*)tableColumn row:(int)row;
- (void)tableDataSource:(BWTableDataSource*)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row;
- (NSDragOperation)tableDataSource:(BWTableDataSource*)source validateDrop:(id<NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)dropOperation;
- (BOOL)tableDataSource:(BWTableDataSource*)source acceptDrop:(id<NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)dropOperation;

- (BOOL)tableDataSource:(BWTableDataSource*)source shouldIncludeRow:(int)row;

- (SEL)tableDataSource:(BWTableDataSource*)source sortSelectorForTableColumn:(NSTableColumn*)tableColumn;

@end

@interface NSObject(BWTableViewDataSourceDelegate)

- (void)tableDataSource:(BWTableDataSource*)source willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row;
- (BOOL)tableDataSource:(BWTableDataSource*)source shouldEditTableColumn:(NSTableColumn *)tableColumn row:(int)row;
- (BOOL)selectionShouldChangeInTableDataSource:(BWTableDataSource*)source;
- (BOOL)tableDataSource:(BWTableDataSource*)source shouldSelectRow:(int)row;
- (BOOL)tableDataSource:(BWTableDataSource*)source shouldSelectTableColumn:(NSTableColumn *)tableColumn;
- (NSColor*)tableDataSource:(BWTableDataSource*)source backgroundColorForRow:(int)row;

- (void)tableDataSource:(BWTableDataSource*)source mouseDownInHeaderOfTableColumn:(NSTableColumn *)tableColumn;
- (void)tableDataSource:(BWTableDataSource*)source didClickTableColumn:(NSTableColumn *)tableColumn;
- (void)tableDataSource:(BWTableDataSource*)source didDragTableColumn:(NSTableColumn *)tableColumn;

- (NSMenu*)tableDataSource:(BWTableDataSource*)source contextMenuForRows:(NSArray*)rows;

- (NSArray*)tableDataSourceAllowedColumnIdentifiers:(BWTableDataSource*)tableDataSource;
- (NSArray*)tableDataSourceDefaultColumnIdentifiers:(BWTableDataSource*)tableDataSource;

- (NSDictionary*)tableDataSource:(BWTableDataSource*)tableDataSource columnInfoForIdentifier:(NSString*)identifier;
- (BOOL)tableDataSource:(BWTableDataSource*)tableDataSource shouldEnableChoiceForColumnIdentifier:(NSString*)identifier;
- (NSString*)tableDataSourceColumnConfigurationName:(BWTableDataSource*)tableDataSource;
- (void)tableDataSource:(BWTableDataSource*)tableDataSource willRemoveColumnWithIdentifier:(NSString*)identifier;
- (void)tableDataSource:(BWTableDataSource*)tableDataSource willAddTableColumn:(NSTableColumn*)tableColumn;

@end
