//
//  CUOSAObject.h
//  UberProject
//
//  Created by Brian Webster on Fri Apr 04 2003.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class AUObjectSpecifier, AUAddress, AUDescriptor, AUList;

@interface CUOSAObject : NSObject <NSCoding>
{
	AUObjectSpecifier*			specifier;
	AUAddress*					address;
	NSMutableDictionary*		cachedProperties;
	NSMutableDictionary*		cachedElements;
}

+ (AUAddress*)defaultAddress;
+ (void)registerScriptModel:(NSDictionary*)model;
+ (AEKeyword)keywordForPropertyKey:(NSString*)key;
+ (NSString*)propertyKeyForKeyword:(AEKeyword)keyword;
+ (NSArray*)propertyKeys;
+ (AEKeyword)keywordForElementKey:(NSString*)key;
+ (NSString*)elementKeyForKeyword:(AEKeyword)keyword;
+ (NSArray*)elementKeys;
+ (Class)classForObjectClass:(FourCharCode)objectClass;
+ (FourCharCode)objectClass;
+ (AUObjectSpecifier*)objectSpecifierForKey:(NSString*)key container:(AUObjectSpecifier*)container;

- (id)initWithObjectSpecifier:(AUObjectSpecifier*)spec;
- (id)initWithObjectSpecifier:(AUObjectSpecifier*)spec address:(AUAddress*)addressSpec;

- (AUObjectSpecifier*)specifier;
- (AUAddress*)address;

- (AEKeyword)keywordForPropertyKey:(NSString*)key;
- (NSString*)propertyKeyForKeyword:(AEKeyword)keyword;
- (NSArray*)propertyKeys;
- (AEKeyword)keywordForElementKey:(NSString*)key;
- (NSString*)elementKeyForKeyword:(AEKeyword)keyword;
- (NSArray*)elementKeys;

- (long)defaultAppleEventTimeout;
- (long)appleEventTimeoutForKey:(NSString*)key;

- (AUDescriptor*)retrieveProperty:(AEKeyword)property;
- (NSString*)retrieveStringProperty:(AEKeyword)property;
- (NSDate*)retrieveDateProperty:(AEKeyword)property;
- (long)retrieveLongProperty:(AEKeyword)property;
- (unsigned long)retrieveUnsignedLongProperty:(AEKeyword)property;
- (short)retrieveShortProperty:(AEKeyword)property;
- (double)retrieveDoubleProperty:(AEKeyword)property;

- (AUList*)retrieveAllElements:(AEKeyword)elementClass;

- (NSArray*)retrieveAllElements:(AEKeyword)elementClass withClass:(Class)objectClass;

- (NSError*)setDescriptor:(AUDescriptor*)inDescriptor forProperty:(AEKeyword)property;
- (NSError*)setString:(NSString*)inString forProperty:(AEKeyword)property;
- (NSError*)setLong:(long)value forProperty:(AEKeyword)property;
- (NSError*)setProperties:(NSDictionary*)properties;

- (id)retrievePropertyValueForKey:(NSString*)key;
- (NSArray*)retrieveAllElementsForKey:(NSString*)key;

- (id)cachedValueForProperty:(NSString*)key;
- (void)cacheValue:(id)value forProperty:(NSString*)key;
- (id)cacheProperty:(NSString*)key;
- (void)cacheAllProperties;
- (void)cacheProperties:(NSArray*)properties;
- (id)missingPropertyValueForKey:(NSString*)key;

- (NSArray*)cachedValueForElement:(NSString*)key;
- (NSMutableArray*)cachedMutableValueForElement:(NSString*)key;

- (unsigned)countOfElement:(NSString*)key;
- (unsigned)cacheCountOfElement:(NSString*)key;

- (void)cacheValue:(id)value atIndex:(unsigned)index inElement:(NSString*)key;
- (id)cacheValueAtIndex:(unsigned)index inElement:(NSString*)key;
- (id)valueAtIndex:(unsigned)index inElement:(NSString*)key;

- (void)cacheValue:(NSArray*)value forElement:(NSString*)key;
- (NSArray*)cacheElement:(NSString*)key;
- (NSArray*)missingElementValueForKey:(NSString*)key;

- (AUObjectSpecifier*)objectSpecifierForKeyPath:(NSString*)keyPath;
- (NSError*)cacheKeyPath:(NSString*)keyPath;
- (id)cachedValueForKeyPath:(NSString*)keyPath;
- (void)cacheValue:(id)value forKeyPath:(NSString*)keyPath;

- (void)uncacheKeyPath:(NSString*)keyPath;
- (void)flushCache;

@end
